@@ -8,19 +8,30 @@ use gtk::{
 
 use std::cell::{Cell, OnceCell, RefCell};
 
-use super::{
-    recognized_page::RecognizedPage, recognizer_status::RecognizerStatus, song_page::SongPage,
-    song_tile::SongTile, AdaptiveMode,
-};
 use crate::{
-    config::APP_ID, i18n::ngettext_f, player::Player, recognizer::Recognizer, song::Song,
-    song_filter::SongFilter, song_list::SongList, song_sorter::SongSorter, uid::Uid, Application,
+    config::APP_ID,
+    i18n::ngettext_f,
+    player::Player,
+    recognizer::Recognizer,
+    song::Song,
+    song_filter::SongFilter,
+    song_list::SongList,
+    song_sorter::SongSorter,
+    uid::Uid,
+    window::{
+        lyrics_page::LyricsPage, recognized_page::RecognizedPage,
+        recognizer_status::RecognizerStatus, song_page::SongPage, song_tile::SongTile,
+        AdaptiveMode,
+    },
+    Application,
 };
 
 // FIXME Missing global navigation shortcuts
 
 const SONG_PAGE_SONG_REMOVE_REQUEST_HANDLER_ID_KEY: &str =
     "mousai-song-page-song-remove-request-handler-id";
+const SONG_PAGE_SHOW_LYRICS_REQUEST_HANDLER_ID_KEY: &str =
+    "mousai-song-page-show-lyrics-request-handler-id";
 const SONG_PAGE_ADAPTIVE_MODE_BINDING_KEY: &str = "mousai-song-page-adaptive-mode-binding";
 
 const RECOGNIZED_PAGE_SONG_ACTIVATED_HANDLER_ID_KEY: &str =
@@ -37,7 +48,7 @@ mod imp {
 
     #[derive(Default, glib::Properties, gtk::CompositeTemplate)]
     #[properties(wrapper_type = super::HistoryView)]
-    #[template(resource = "/io/github/seadve/Mousai/ui/history-view.ui")]
+    #[template(resource = "/io/github/seadve/Mousai/ui/history_view.ui")]
     pub struct HistoryView {
         /// Whether selection mode is active
         #[property(get)]
@@ -94,7 +105,7 @@ mod imp {
 
     #[glib::object_subclass]
     impl ObjectSubclass for HistoryView {
-        const NAME: &'static str = "MsaiHistoryView";
+        const NAME: &'static str = "MousaiHistoryView";
         type Type = super::HistoryView;
         type ParentType = gtk::Widget;
 
@@ -129,7 +140,7 @@ mod imp {
 
                 let text = selected_songs
                     .iter()
-                    .map(|song| song.copy_term())
+                    .map(|song| song.artist_title_text())
                     .collect::<Vec<_>>()
                     .join("\n");
                 obj.display().clipboard().set_text(&text);
@@ -167,8 +178,10 @@ mod imp {
 
             let obj = self.obj();
 
-            self.navigation_view
-                .connect_pushed(clone!(@weak obj => move |view| {
+            self.navigation_view.connect_pushed(clone!(
+                #[weak]
+                obj,
+                move |view| {
                     let imp = obj.imp();
 
                     let visible_page = view
@@ -188,25 +201,28 @@ mod imp {
                             }
                         }
                     }
-                }));
-            self.navigation_view
-                .connect_popped(clone!(@weak obj => move |_, page| {
+                }
+            ));
+            self.navigation_view.connect_popped(clone!(
+                #[weak]
+                obj,
+                move |_, page| {
                     obj.imp()
                         .navigation_forward_stack
                         .borrow_mut()
                         .push(page.clone());
-                }));
-            self.navigation_view.connect_get_next_page(
-                clone!(@weak obj => @default-panic, move |_| {
-                    let next_page = obj.imp()
-                        .navigation_forward_stack
-                        .borrow()
-                        .last()
-                        .cloned();
+                }
+            ));
+            self.navigation_view.connect_get_next_page(clone!(
+                #[weak]
+                obj,
+                #[upgrade_or_panic]
+                move |_| {
+                    let next_page = obj.imp().navigation_forward_stack.borrow().last().cloned();
 
                     next_page
-                }),
-            );
+                }
+            ));
 
             self.content_empty_page.set_icon_name(Some(APP_ID));
             obj.setup_grid();
@@ -281,11 +297,13 @@ impl HistoryView {
         unsafe {
             recognized_page.set_data(
                 RECOGNIZED_PAGE_SONG_ACTIVATED_HANDLER_ID_KEY,
-                recognized_page.connect_song_activated(
-                    clone!(@weak self as obj => move |_, song| {
+                recognized_page.connect_song_activated(clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |_, song| {
                         obj.push_song_page(song);
-                    }),
-                ),
+                    }
+                )),
             );
             recognized_page.set_data(
                 RECOGNIZED_PAGE_ADAPTIVE_MODE_BINDING_KEY,
@@ -303,14 +321,20 @@ impl HistoryView {
         let imp = self.imp();
 
         // Return if the last widget is a `SongPage` and its song's id is the same as the given song's id
-        if let Some(visible_page) = imp.navigation_view.visible_page() {
-            if let Some(song_page) = visible_page.downcast_ref::<SongPage>() {
-                if let Some(song_page_song) = song_page.song() {
-                    if song_page_song.id_ref() == song.id_ref() {
-                        return;
-                    }
-                }
-            }
+        if imp
+            .navigation_view
+            .visible_page()
+            .is_some_and(|visible_page| {
+                visible_page
+                    .downcast_ref::<SongPage>()
+                    .is_some_and(|song_page| {
+                        song_page
+                            .song()
+                            .is_some_and(|s| s.id_ref() == song.id_ref())
+                    })
+            })
+        {
+            return;
         }
 
         let song_page = SongPage::new();
@@ -321,15 +345,37 @@ impl HistoryView {
         unsafe {
             song_page.set_data(
                 SONG_PAGE_SONG_REMOVE_REQUEST_HANDLER_ID_KEY,
-                song_page.connect_song_remove_request(clone!(@weak self as obj => move |_, song| {
-                    if let Err(err) = obj.remove_songs(&[song.id_ref()]) {
-                        tracing::error!("Failed to remove song: {:?}", err);
-                        Application::get().add_message_toast(&gettext("Failed to remove song"));
-                        return;
-                    }
+                song_page.connect_song_remove_request(clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |song_page| {
+                        let song = song_page
+                            .song()
+                            .expect("song page must have a song on request");
 
-                    obj.show_undo_remove_song_toast();
-                })),
+                        if let Err(err) = obj.remove_songs(&[song.id_ref()]) {
+                            tracing::error!("Failed to remove song: {:?}", err);
+                            Application::get().add_message_toast(&gettext("Failed to remove song"));
+                            return;
+                        }
+
+                        obj.show_undo_remove_song_toast();
+                    }
+                )),
+            );
+            song_page.set_data(
+                SONG_PAGE_SHOW_LYRICS_REQUEST_HANDLER_ID_KEY,
+                song_page.connect_show_lyrics_request(clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |song_page| {
+                        let song = song_page
+                            .song()
+                            .expect("song page must have a song on request");
+
+                        obj.push_lyrics_page(&song);
+                    }
+                )),
             );
             song_page.set_data(
                 SONG_PAGE_ADAPTIVE_MODE_BINDING_KEY,
@@ -345,6 +391,54 @@ impl HistoryView {
         song.set_is_newly_heard(false);
     }
 
+    /// Pushes a `LyricsPage` for the given song to the navigation stack.
+    pub fn push_lyrics_page(&self, song: &Song) {
+        let imp = self.imp();
+
+        // Return if the last widget is a `LyricsPage` and its song's id is the same as the given song's id
+        if imp
+            .navigation_view
+            .visible_page()
+            .is_some_and(|visible_page| {
+                visible_page
+                    .downcast_ref::<LyricsPage>()
+                    .is_some_and(|lyrics_page| {
+                        lyrics_page
+                            .song()
+                            .is_some_and(|s| s.id_ref() == song.id_ref())
+                    })
+            })
+        {
+            return;
+        }
+
+        // If the next page is a `LyricsPage` with the same song id, just navigate to it.
+        //
+        // This is done to restore the scroll position of the `LyricsPage` when navigating back to it.
+        let forward_page = imp.navigation_forward_stack.borrow().last().cloned();
+        if let Some(forward_page) = forward_page {
+            if forward_page
+                .downcast_ref::<LyricsPage>()
+                .is_some_and(|lyrics_page| {
+                    lyrics_page
+                        .song()
+                        .is_some_and(|s| s.id_ref() == song.id_ref())
+                })
+            {
+                let removed = imp.navigation_forward_stack.borrow_mut().pop();
+                debug_assert_eq!(removed.as_ref(), Some(&forward_page));
+
+                imp.navigation_view.push(&forward_page);
+                return;
+            }
+        }
+
+        let lyrics_page = LyricsPage::new();
+        lyrics_page.set_song(Some(song));
+
+        imp.navigation_view.push(&lyrics_page);
+    }
+
     /// Returns true if a page has been popped
     pub fn pop_page(&self) -> bool {
         self.imp().navigation_view.pop()
@@ -359,63 +453,89 @@ impl HistoryView {
     pub fn bind_song_list(&self, song_list: &SongList) {
         let imp = self.imp();
 
-        song_list.connect_items_changed(clone!(@weak self as obj => move |_, _, _, _| {
-            obj.update_content_stack_visible_child();
-        }));
+        song_list.connect_items_changed(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            move |_, _, _, _| {
+                obj.update_content_stack_visible_child();
+            }
+        ));
 
         let filter = SongFilter::new();
         let sorter = SongSorter::new();
 
         let filter_model = gtk::FilterListModel::new(Some(song_list.clone()), Some(filter.clone()));
-        filter_model.connect_items_changed(clone!(@weak self as obj => move |_, _, _, _| {
-            obj.update_content_stack_visible_child();
-        }));
+        filter_model.connect_items_changed(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            move |_, _, _, _| {
+                obj.update_content_stack_visible_child();
+            }
+        ));
 
-        imp.search_entry.connect_search_changed(
-            clone!(@weak self as obj, @weak filter, @weak sorter => move |search_entry| {
+        imp.search_entry.connect_search_changed(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            #[weak]
+            filter,
+            #[weak]
+            sorter,
+            move |search_entry| {
                 let text = search_entry.text();
                 filter.set_search(text.trim());
                 sorter.set_search(text.trim());
                 obj.update_content_stack_visible_child();
-            }),
-        );
+            }
+        ));
 
         let sort_model = gtk::SortListModel::new(Some(filter_model.clone()), Some(sorter));
 
         // FIXME save selection even when the song are filtered from FilterListModel
         let selection_model = gtk::MultiSelection::new(Some(sort_model));
-        selection_model.connect_selection_changed(clone!(@weak self as obj => move |model, _, _| {
-            if obj.is_selection_mode_active() {
-                if model.selection().size() == 0 {
-                    obj.set_selection_mode_active(false);
-                }
+        selection_model.connect_selection_changed(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            move |model, _, _| {
+                if obj.is_selection_mode_active() {
+                    if model.selection().size() == 0 {
+                        obj.set_selection_mode_active(false);
+                    }
 
-                obj.update_selection_actions();
-            }
-        }));
-        selection_model.connect_items_changed(clone!(@weak self as obj => move |model, _, _, _| {
-            if obj.is_selection_mode_active() {
-                if model.selection().size() == 0 {
-                    obj.set_selection_mode_active(false);
+                    obj.update_selection_actions();
                 }
-
-                obj.update_selection_actions();
             }
-        }));
+        ));
+        selection_model.connect_items_changed(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            move |model, _, _, _| {
+                if obj.is_selection_mode_active() {
+                    if model.selection().size() == 0 {
+                        obj.set_selection_mode_active(false);
+                    }
+
+                    obj.update_selection_actions();
+                }
+            }
+        ));
 
         let grid = imp.grid.get();
         grid.set_model(Some(&selection_model));
-        grid.connect_activate(
-            clone!(@weak self as obj, @weak selection_model => move |_, index| {
+        grid.connect_activate(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            #[weak]
+            selection_model,
+            move |_, index| {
                 match selection_model.item(index) {
                     Some(ref item) => {
                         let song = item.downcast_ref::<Song>().unwrap();
                         obj.push_song_page(song);
                     }
-                    None => unreachable!("selection model must have item at index `{}`", index)
+                    None => unreachable!("selection model must have item at index `{}`", index),
                 }
-            }),
-        );
+            }
+        ));
 
         imp.song_list.set(song_list.downgrade()).unwrap();
         imp.filter_model.set(filter_model.downgrade()).unwrap();
@@ -432,15 +552,19 @@ impl HistoryView {
 
         imp.recognizer_status.bind_recognizer(recognizer);
 
-        imp.recognizer_status.connect_show_results_requested(
-            clone!(@weak self as obj, @weak recognizer => move |_| {
+        imp.recognizer_status.connect_show_results_requested(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            #[weak]
+            recognizer,
+            move |_| {
                 if let Err(err) = obj.show_recognizer_results(&recognizer) {
                     tracing::error!("Failed to show recognizer results: {:?}", err);
                     Application::get()
                         .add_message_toast(&gettext("Failed to show recognizer results"));
                 }
-            }),
-        );
+            }
+        ));
     }
 
     pub fn scroll_to_top(&self) -> bool {
@@ -547,21 +671,29 @@ impl HistoryView {
                 .button_label(gettext("_Undo"))
                 .build();
 
-            toast.connect_button_clicked(clone!(@weak self as obj => move |_| {
-                if let Err(err) = obj
-                    .song_list()
-                    .insert_many(obj.imp().songs_purgatory.take())
-                {
-                    tracing::error!("Failed to undo remove song: {:?}", err);
-                    Application::get().add_message_toast(&gettext("Failed to undo"));
+            toast.connect_button_clicked(clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| {
+                    if let Err(err) = obj
+                        .song_list()
+                        .insert_many(obj.imp().songs_purgatory.take())
+                    {
+                        tracing::error!("Failed to undo remove song: {:?}", err);
+                        Application::get().add_message_toast(&gettext("Failed to undo"));
+                    }
                 }
-            }));
+            ));
 
-            toast.connect_dismissed(clone!(@weak self as obj => move |_| {
-                let imp = obj.imp();
-                imp.songs_purgatory.borrow_mut().clear();
-                imp.undo_remove_song_toast.take();
-            }));
+            toast.connect_dismissed(clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| {
+                    let imp = obj.imp();
+                    imp.songs_purgatory.borrow_mut().clear();
+                    imp.undo_remove_song_toast.take();
+                }
+            ));
 
             Application::get().add_toast(toast.clone());
 
@@ -725,69 +857,88 @@ impl HistoryView {
 
     fn setup_grid(&self) {
         let factory = gtk::SignalListItemFactory::new();
-        factory.connect_setup(clone!(@weak self as obj => move |_, list_item| {
-            let list_item = list_item.downcast_ref::<gtk::ListItem>().unwrap();
+        factory.connect_setup(clone!(
+            #[weak(rename_to = obj)]
+            self,
+            move |_, list_item| {
+                let list_item = list_item.downcast_ref::<gtk::ListItem>().unwrap();
 
-            let song_tile = SongTile::new();
-            song_tile.set_shows_select_button_on_hover(true);
-            song_tile.bind_player(&obj.player());
+                let song_tile = SongTile::new();
+                song_tile.set_shows_select_button_on_hover(true);
+                song_tile.bind_player(&obj.player());
 
-            let selection_mode_active_binding = obj
-                .bind_property("is-selection-mode-active", &song_tile, "is-selection-mode-active")
-                .sync_create()
-                .build();
-            let adaptive_mode_binding = obj
-                .bind_property("adaptive-mode", &song_tile, "adaptive-mode")
-                .sync_create()
-                .build();
+                let selection_mode_active_binding = obj
+                    .bind_property(
+                        "is-selection-mode-active",
+                        &song_tile,
+                        "is-selection-mode-active",
+                    )
+                    .sync_create()
+                    .build();
+                let adaptive_mode_binding = obj
+                    .bind_property("adaptive-mode", &song_tile, "adaptive-mode")
+                    .sync_create()
+                    .build();
 
-            song_tile.connect_is_active_notify(clone!(@weak obj, @weak list_item => move |tile| {
-                let selection_model = obj
-                    .imp()
-                    .selection_model
-                    .get()
-                    .and_then(|model| model.upgrade())
-                    .expect("selection model should exist");
-                if tile.is_active() {
-                    selection_model.select_item(list_item.position(), false);
-                } else {
-                    selection_model.unselect_item(list_item.position());
+                song_tile.connect_is_active_notify(clone!(
+                    #[weak]
+                    obj,
+                    #[weak]
+                    list_item,
+                    move |tile| {
+                        let selection_model = obj
+                            .imp()
+                            .selection_model
+                            .get()
+                            .and_then(|model| model.upgrade())
+                            .expect("selection model should exist");
+                        if tile.is_active() {
+                            selection_model.select_item(list_item.position(), false);
+                        } else {
+                            selection_model.unselect_item(list_item.position());
+                        }
+                    }
+                ));
+                song_tile.connect_selection_mode_requested(clone!(
+                    #[weak]
+                    obj,
+                    move |_| {
+                        obj.set_selection_mode_active(true);
+                    }
+                ));
+
+                let song_watch = list_item.property_expression("item").bind(
+                    &song_tile,
+                    "song",
+                    glib::Object::NONE,
+                );
+                let selected_watch = gtk::ClosureExpression::new::<bool>(
+                    [
+                        list_item.property_expression("selected"),
+                        obj.property_expression("is-selection-mode-active"),
+                    ],
+                    closure!(|_: Option<glib::Object>,
+                              is_selected: bool,
+                              is_selection_mode_active: bool| {
+                        is_selected && is_selection_mode_active
+                    }),
+                )
+                .bind(&song_tile, "is-selected", glib::Object::NONE);
+
+                unsafe {
+                    list_item.set_data(
+                        GRID_LIST_ITEM_BINDINGS_KEY,
+                        vec![selection_mode_active_binding, adaptive_mode_binding],
+                    );
+                    list_item.set_data(
+                        GRID_LIST_ITEM_EXPRESSION_WATCHES_KEY,
+                        vec![song_watch, selected_watch],
+                    );
                 }
-            }));
-            song_tile.connect_selection_mode_requested(clone!(@weak obj => move |_| {
-                obj.set_selection_mode_active(true);
-            }));
 
-            let song_watch =
-                list_item
-                    .property_expression("item")
-                    .bind(&song_tile, "song", glib::Object::NONE);
-            let selected_watch = gtk::ClosureExpression::new::<bool>(
-                [
-                    list_item.property_expression("selected"),
-                    obj.property_expression("is-selection-mode-active"),
-                ],
-                closure!(|_: Option<glib::Object>,
-                        is_selected: bool,
-                        is_selection_mode_active: bool| {
-                    is_selected && is_selection_mode_active
-                }),
-            )
-            .bind(&song_tile, "is-selected", glib::Object::NONE);
-
-            unsafe {
-                list_item.set_data(
-                    GRID_LIST_ITEM_BINDINGS_KEY,
-                    vec![selection_mode_active_binding, adaptive_mode_binding],
-                );
-                list_item.set_data(
-                    GRID_LIST_ITEM_EXPRESSION_WATCHES_KEY,
-                    vec![song_watch, selected_watch],
-                );
+                list_item.set_child(Some(&song_tile));
             }
-
-            list_item.set_child(Some(&song_tile));
-        }));
+        ));
         factory.connect_teardown(|_, list_item| {
             let list_item = list_item.downcast_ref::<gtk::ListItem>().unwrap();
 
@@ -812,13 +963,19 @@ impl HistoryView {
     }
 }
 
+/// This undoes the bindings and connections done on `push_*_page` methods.
 fn unbind_page(page: &adw::NavigationPage) {
     if let Some(song_page) = page.downcast_ref::<SongPage>() {
         unsafe {
-            let handler_id = song_page
+            let song_remove_request_handler_id = song_page
                 .steal_data::<glib::SignalHandlerId>(SONG_PAGE_SONG_REMOVE_REQUEST_HANDLER_ID_KEY)
                 .unwrap();
-            song_page.disconnect(handler_id);
+            song_page.disconnect(song_remove_request_handler_id);
+
+            let show_lyrics_request_handler_id = song_page
+                .steal_data::<glib::SignalHandlerId>(SONG_PAGE_SHOW_LYRICS_REQUEST_HANDLER_ID_KEY)
+                .unwrap();
+            song_page.disconnect(show_lyrics_request_handler_id);
 
             let binding = song_page
                 .steal_data::<glib::Binding>(SONG_PAGE_ADAPTIVE_MODE_BINDING_KEY)
@@ -840,6 +997,8 @@ fn unbind_page(page: &adw::NavigationPage) {
             adaptive_mode_binding.unbind();
         }
         recognized_page.unbind_player();
+    } else if page.downcast_ref::<LyricsPage>().is_some() {
+        // Nothing to unbind
     } else {
         unreachable!(
             "tried to unbind unknown navigation page type `{}`",
@@ -848,12 +1007,6 @@ fn unbind_page(page: &adw::NavigationPage) {
     }
 }
 
-impl Default for HistoryView {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
 #[cfg(test)]
 mod test {
     use super::*;
